@@ -6,12 +6,10 @@ from sql import Table
 from sql.functions import Overlay, Position
 
 from trytond.model import ModelView, ModelSQL
-from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
-__all__ = ['ShipmentInternal',
-    'CreateShipmentInternalStart', 'CreateShipmentInternal']
+__all__ = ['ShipmentInternal']
 
 
 class ShipmentInternal(ModelSQL, ModelView):
@@ -40,10 +38,12 @@ class ShipmentInternal(ModelSQL, ModelView):
         super(ShipmentInternal, cls).__register__(module_name)
 
     @classmethod
-    def generate_internal_shipment(cls):
+    def generate_internal_shipment(cls, clean=True):
         """
         Generate internal shipments to meet order points defined on
         non-warehouse location.
+
+        If clean is set, it will remove all previous requests.
         """
         pool = Pool()
         OrderPoint = pool.get('stock.order_point')
@@ -57,6 +57,13 @@ class ShipmentInternal(ModelSQL, ModelView):
         user_record = User(Transaction().user)
         today = Date.today()
         lead_time = LeadTime.get_max_lead_time()
+
+        if clean:
+            reqs = cls.search([
+                    ('state', '=', 'request'),
+                    ])
+            cls.delete(reqs)
+
         # fetch quantities on order points
         order_points = OrderPoint.search([
             ('type', '=', 'internal'),
@@ -70,13 +77,15 @@ class ShipmentInternal(ModelSQL, ModelView):
                 (op.storage_location.id, op.product.id)
                 ] = op
             id2location[op.storage_location.id] = op.storage_location
-        provisioned = Location.search([
+        implicit_locations = Location.search(['OR',
                 ('provisioning_location', '!=', None),
+                ('overflowing_location', '!=', None),
                 ])
-        id2location.update({l.id: l for l in provisioned})
+        id2location.update({l.id: l for l in implicit_locations})
+        location_ids = id2location.keys()
 
         # ordered by ids to speedup reduce_ids in products_by_location
-        if provisioned:
+        if implicit_locations:
             products = Product.search([
                     ('type', 'in', ['goods', 'assets']),
                     ], order=[('id', 'ASC')])
@@ -85,32 +94,52 @@ class ShipmentInternal(ModelSQL, ModelView):
             product_ids = id2product.keys()
             product_ids.sort()
 
+        with Transaction().set_context(forecast=True, stock_date_end=today):
+            pbl = Product.products_by_location(
+                location_ids, product_ids, with_childs=True)
+
         shipments = []
         date = today
         end_date = date + lead_time
+        current_qties = pbl.copy()
         while date <= end_date:
-            with Transaction().set_context(forecast=True, stock_date_end=date):
-                pbl = Product.products_by_location(id2location.keys(),
-                    product_ids, with_childs=True)
-
             # Create a list of moves to create
             moves = {}
             for location in id2location.itervalues():
                 for product_id in product_ids:
-                    qty = pbl.get((location.id, product_id), 0)
+                    qty = current_qties.get((location.id, product_id), 0)
                     op = product2op.get((location.id, product_id))
                     if op:
                         min_qty, max_qty = op.min_quantity, op.max_quantity
-                        provisioning_location = op.provisioning_location
-                    elif location and location.provisioning_location:
-                        min_qty, max_qty = 0, 0
-                        provisioning_location = location.provisioning_location
+                        target_qty = op.target_quantity
+                        prov_location = op.provisioning_location
+                        over_location = op.overflowing_location
+                    elif (location
+                            and (location.provisioning_location
+                                or location.overflowing_location)):
+                        target_qty = 0
+                        min_qty = 0 if location.provisioning_location else None
+                        max_qty = 0 if location.overflowing_location else None
+                        prov_location = location.provisioning_location
+                        over_location = location.overflowing_location
                     else:
                         continue
-                    if qty < min_qty:
-                        key = (
-                            provisioning_location.id, location.id, product_id)
-                        moves[key] = max_qty - qty
+
+                    change_qty = 0
+                    if min_qty is not None and qty < min_qty:
+                        from_loc = prov_location.id
+                        to_loc = location.id
+                        change_qty = target_qty - qty
+                    elif max_qty is not None and qty > max_qty:
+                        from_loc = location.id
+                        to_loc = over_location.id
+                        change_qty = qty - target_qty
+
+                    if change_qty:
+                        key = (from_loc, to_loc, product_id)
+                        moves[key] = change_qty
+                        current_qties[(from_loc, product_id)] -= change_qty
+                        current_qties[(to_loc, product_id)] += change_qty
 
             # Group moves by {from,to}_location
             to_create = {}
@@ -125,6 +154,7 @@ class ShipmentInternal(ModelSQL, ModelView):
                     from_location=from_location,
                     to_location=to_location,
                     planned_date=date,
+                    state='request',
                     )
                 shipment_moves = []
                 for move in moves:
@@ -145,30 +175,19 @@ class ShipmentInternal(ModelSQL, ModelView):
                     shipment.on_change_with_planned_start_date())
                 shipments.append(shipment)
             date += datetime.timedelta(1)
-        cls.save(shipments)
-        cls.wait(shipments)
+
+            # Update quantities with next moves
+            with Transaction().set_context(
+                    forecast=True,
+                    stock_date_start=date,
+                    stock_date_end=date):
+                pbl = Product.products_by_location(
+                    location_ids, product_ids, with_childs=True)
+            for key, qty in pbl.iteritems():
+                current_qties[key] += qty
+
+        if shipments:
+            cls.save(shipments)
+            # Split moves through transit to get accurate dates
+            cls._set_transit(shipments)
         return shipments
-
-
-class CreateShipmentInternalStart(ModelView):
-    'Create Shipment Internal'
-    __name__ = 'stock.shipment.internal.create.start'
-
-
-class CreateShipmentInternal(Wizard):
-    'Create Shipment Internal'
-    __name__ = 'stock.shipment.internal.create'
-    start = StateView('stock.shipment.internal.create.start',
-        'stock_supply.shipment_internal_create_start_view_form', [
-            Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Create', 'create_', 'tryton-ok', default=True),
-            ])
-    create_ = StateAction('stock.act_shipment_internal_form')
-
-    def do_create_(self, action):
-        ShipmentInternal = Pool().get('stock.shipment.internal')
-        ShipmentInternal.generate_internal_shipment()
-        return action, {}
-
-    def transition_create_(self):
-        return 'end'
